# [User Manual](@id manual)

Here we document key functionalities offered in `ClimateModels.jl`

- Climate Model Interface
- Tracked Worklow Framework
- Cloud + On-Premise File Support

## Climate Model Interface

The climate model interface is based on the [`ModelConfig`](@ref) data structure and a series of methods like [`setup`](@ref), [`build`](@ref), and [`launch`](@ref). The typical sequence is shown just below where `f` is a function that (1) receives a `ModelConfig` as its only input argument, and (2) gets called via [`launch`](@ref). 

```julia
julia> using ClimateModels
julia> f=ClimateModels.RandomWalker
julia> MC=ModelConfig(model=f)
julia> setup(MC)
julia> build(MC)
julia> launch(MC)
```

The `ModelConfig` called `MC` is summarized using the [`show`](@ref) method which here reveals that `f` is just an alias for `ClimateModels.RandomWalker`. 

```
  ID            = 6e8444b3-220f-402e-9224-dbd11b513790
  model         = RandomWalker
  configuration = anonymous
  run folder    = /tmp/6e8444b3-220f-402e-9224-dbd11b513790
  log subfolder = /tmp/6e8444b3-220f-402e-9224-dbd11b513790/log
  task(s)       = RandomWalker
```

The run folder name can be accessed directly using [`pathof`](@ref) and its content inspected using [`readdir`](@ref).

```
2-element Vector{String}:
 "RandomWalker.csv"
 "log"
```

Here `launch(MC)` has completed, `RandomWalker.csv` is a file that was generated by function `f`, and `log` is the log subfolder that was created by [`setup`](@ref). 

The workflow log is retrieved using the [`log`](@ref) function. `log(MC)` in this example would give:

```
5-element Vector{String}:
 "9947f79 initial setup"
 "63e7e77 add Project.toml to log"
 "fa2a84e add Manifest.toml to log"
 "89359f5 task started [64e58e36-303a-4903-8ecd-2270f6583f0b]"
 "63e8857 (HEAD -> main) task ended [64e58e36-303a-4903-8ecd-2270f6583f0b]"
```

This highlights that `Project.toml` and `Manifest.toml` for the environment being used have been archived. This happens during [`setup`](@ref) to document all dependencies and make the workflow reproducible.

### Generalization

A key point is that everything can be customized to, e.g., use popular models previously written in Fortran or C just as simply. This typically involves defining a new concrete type of `AbstractModelConfig` and then providing customized `build` and/or `setup` methods as discussed below. 

To start, let's distinguish amongst [`ModelConfig`](@ref)s on the basis of their `model` variable type :

- _normal user mode_ is when `model` is a `String` or a `Function`
- _package developer mode_ is when `model` is a `Pkg.Types.PackageSpec`

#### _Normal User Mode_

The simplest way to use the `ClimateModels.jl` interface is to specify `model` directly as a function, and use defaults for everything else. This approach may be best suited to pure Julia models or as a first step. It is illustrated in [random walk](../examples/RandomWalker.html). In [CMIP6](../examples/CMIP6.html) the `model` name is provided as a `String` and the main `Function` is specified as the `configuration` instead.

!!! note
    Once the initial [`launch`](@ref) call has completed, it is always possible to add workflow steps via [`put!`](@ref) and [`launch`](@ref).

Often though, there are benefits to defining a custom `setup` and/or `build` method. One can then simply define a concrete type of `AbstractModelConfig` using [`ModelConfig`](@ref) as a blueprint. This is the recommended approach when another languange like Fortran, C++, or Python is involved. Thiis is illustrated with [Hector](../examples/Hector.html), [FaIR](../examples/FaIR.html), [SPEEDY](../examples/Speedy.html), and [MITgcm](../examples/MITgcm.html). 

Defining a concrete type of `AbstractModelConfig` can also be practical with pure Julia model, e.g. to speed up [`launch`](@ref), generate ensembles, facilitate checkpointing, etc. That's the case in the [Oceananigans.jl](../examples/Oceananigans.html) example.

The idea in the longer term is that for popular models the customized interface elements would be provided via a dedicated package. They would thus be maintained independently by developers and users most familiar with each model. This approach is demonstrated in [MITgcmTools.jl](https://github.com/gaelforget/MITgcmTools.jl) for [MITgcm](https://mitgcm.readthedocs.io/en/latest/) which provides its own suite of examples using the `ClimateModels.jl` interface.

#### _Package Developer Mode_

The defining feature of this approach is that the `PackageSpec`   specification of `model` makes [`setup`](@ref) install the chosen package using `Pkg.develop`. This allows for developing a package or using an unregistered package in the context of `ClimateModels.jl`. There are two cases: 

- if `configuration` is left undefined then `launch` will run the package test suite using `Pkg.test` as in [this example](../examples/defaults.html) ([code link](https://raw.githubusercontent.com/gaelforget/ClimateModels.jl/master/examples/defaults.jl), [download link](defaults.jl))
- if `configuration` is provided as a `Function` then `launch` will call it as illustrated in the [ShallowWaters.jl example](../examples/ShallowWaters.html) ([code link](https://raw.githubusercontent.com/gaelforget/ClimateModels.jl/master/examples/ShallowWaters.jl), [download link](ShallowWaters.jl))

!!! note 
    As an exercise, can you turn [ShallowWaters.jl example](../examples/ShallowWaters.html) into a _normal user mode_ example?

## Git / Log Support

The `setup` method normally calls [`log`](@ref) to create a temporary run folder with a `git` enabled subfolder called `log`. This allows for recording each workflow step, using [`log`](@ref) functions listed below.

```@docs
log
```

## Cloud / File Support

There are various ways that numerical model output gets archived, distributed, and retrieved from the internet. In some cases downloading data can be the most convenient approach. In others it can be more advantageous to compute in the cloud and only download final results for plotting. 

`ClimateModels.jl` comes equiped with packages that read popular file formats used in climate modeling and science. [Downloads.jl](https://github.com/JuliaLang/Downloads.jl), [CSV.jl](https://github.com/JuliaData/CSV.jl), [DataFrames.jl](https://github.com/JuliaData/DataFrames.jl), [NetCDF.jl](https://github.com/JuliaGeo/NetCDF.jl), [Zarr.jl](https://github.com/meggart/Zarr.jl), and [TOML.jl](https://github.com/JuliaLang/TOML.jl) are thus readily available when you install `ClimateModels.jl`. For instance, one can read the CSV file generated before as

```julia
julia> fil=joinpath(pathof(MC),"RandomWalker.csv")
julia> ClimateModels.CSV.File(fil) |> ClimateModels.DataFrame
```

or, maybe preferably, as

```
julia> fil=joinpath(pathof(MC),"RandomWalker.csv")
julia> CSV=ClimateModels.CSV
julia> DataFrame=ClimateModels.DataFrame
julia> CSV.File(fil) |> DataFrame
```

For additional examples covering other file formats, please refer to the [IPCC report](../examples/IPCC.html) and [CMIP6 archive](../examples/CMIP6.html) notebooks and code links.

## API Reference

```@index
```

```@docs
ModelConfig
```

```@docs
setup
build
launch
show
pathof
readdir
clean
```

```@docs
cmip
```
